#!/usr/bin/env python

# Rss Notify
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 3
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful, 
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
# 02110-1301, USA.
#
# Authors: Isaiah Heyer (freshapplepy@gmail.com)

# Copyright (C) 2009 Isaiah Heyer

import feedparser
import os
import gobject

import gtk
import rss_notify

from xml.etree import ElementTree as ET
from xml.dom.minidom import parseString


class FeedManager(gobject.GObject):

    ## register a signal that is emitted whenever feeds
    ## are added or removed.
    __gsignals__ = {
            "feed-list-changed": (gobject.SIGNAL_RUN_FIRST,
                    gobject.TYPE_NONE, [])
                    }

    def __init__(self, opml_filename):

        gobject.GObject.__init__(self)

        self.opml_file = opml_filename
        self.tree = None


        if os.path.exists(self.opml_file):
            
            self.import_feeds(self.opml_file)

        else:
            self.tree = self.build_tree()


        feed_icon = os.path.join(rss_notify.IMAGE_PATH,
                "standard.svg")

        self.feed_pixbuf = gtk.gdk.pixbuf_new_from_file_at_size(
                feed_icon, 16, 16)

        dummy = gtk.Button()
        self.dir_pixbuf = dummy.render_icon(gtk.STOCK_DIRECTORY,
                        gtk.ICON_SIZE_MENU)



    def build_tree(self):
        """Builds a skeleton of an ElementTree.
        """

        root = ET.Element("opml", {"version":"1.0"})

        head = ET.SubElement(root, "head")
        title = ET.SubElement(head, "title")
        title.text = "Rss Notify Feed List"

        body = ET.SubElement(root, "body")

        tree = ET.ElementTree(root)

        return tree
    
    
    
    ### creates a feed element but does not add
    ### it to the ElementTree.

    def create_feed(self, xmlUrl):
        
        xmlUrl = xmlUrl.strip()
        
        try:
            f = feedparser.parse(xmlUrl)

            title = f.feed.title
            text = f.feed.title
            description = f.feed.title
            otype = "rss"
            htmlUrl = f.feed.link
            xmlUrl = f.url

        except Exception, data:
            print data
            #import traceback
            #print traceback.print_exc()
            return False


                
        new_feed_info = {"title":title, "text":text, "description":description,
                "type":otype, "htmlUrl":htmlUrl, "xmlUrl":xmlUrl}

        elem = ET.Element("outline", new_feed_info)

        return elem
    
    def move_feed(self, elem, category):
        
        self.remove_feed(elem)

        if category == None:
            body = self.get_feed_outlines()
            body.append(elem)
        else:
            category.append(elem)

    def add_feed(self, xmlUrl, category=None):
        
        e = self.create_feed(xmlUrl)
        
        ### add new feed to category
        if category != None:
            category.append(e)
        else:
            body = self.get_feed_outlines()
            body.append(e)

        self.emit("feed-list-changed")
        return e

    
    def create_category(self, name):

        title = name
        text = name
        description = name
        otype = "folder"
        
        new_cat_info = {"title":title, "text":text,
                "description":description, "type":otype}

        e = ET.Element("outline", new_cat_info)

        self.cat_key += 1
        self.cat_elems[self.cat_key] = e

        return e


    def move_category(self, key, parent_key):
        pass

    def add_category(self, name, parent):
        """Adds a new feed category.
        """
        
        e = self.create_category(name)
        pname = self._get_name(parent)

        if parent != None:
            parent.append(e)
        else:
            body = self.get_feed_outlines()
            body.append(e)

        return e


    def remove_feed(self, rem_element):
        """Removes rem_element from internal feed list
        """

        root = self.tree.getroot()

        def trav(element, remove_element):
            if remove_element in element.getchildren():
                element.remove(remove_element)
            else:
                for child in element.getchildren():
                    trav(child, remove_element)

        trav(root, rem_element)

        self.emit("feed-list-changed")
        return True


    def get_feed_urls(self):
        """Returns a list of all feeds urls"""
        Url_list = []

        def trav(element):
            xmlUrl = element.get("xmlUrl", None)
            if xmlUrl:
                    Url_list.append(xmlUrl)
            for child in element:
                trav(child)

        gen = trav(self.tree.getroot())
        
        return Url_list

    def get_feed_outlines(self):
        """Returns the <body> element which contains all the 
        <opml> elements.
        """

        for child in self.tree.getroot():
            if child.tag == "body":
                return child

    

    def get_category(self, key):
        return self.cat_elems[key]

    def get_feed(self, key):
        return self.feed_elems[key]
    
    
    def populate_cbb_liststore(self, liststore):
        
        #self.update_treeview()
        category_rows = []

        def trav(element, pname=""):


            if element.get("type", None) == "folder":
                if pname:
                    pname = pname+" > "+ self._get_name(element)
                else:
                    pname = self._get_name(element)

                row = [self.dir_pixbuf, pname, self.cat_key]

                category_rows.append(row)

            for child in element:
                trav(child, pname)

        root = self.tree.getroot()
        trav(root)
        liststore.clear()

        for row in category_rows:
            liststore.append(row)


    def set_treeview(self, treeview):

        self.treeview = treeview

    def _get_name(self, element):
        try:
            keys = ["text", "title", "description"]
            for k in keys:
                name = element.get(k, None)
                if name == None:
                    continue
                return name
        except Exception, data:
            print data
            return ""

    def update_treeview(self):
        
        treestore = self.treeview.get_model()

        ### re-expand rows
        expanded_rows = []
        def for_expanded(treeview, path, data=None):
            expanded_rows.append(path)

        self.treeview.map_expanded_rows(for_expanded, None)


        self.feed_key = 1
        self.feed_elems = {}

        self.cat_key = 1
        self.cat_elems = {}


        self.category_rows = []
        
        treestore.clear()
    

        def trav(element, piter=None):
            
            if element.get('type') == "rss":

                pixbuf = self.feed_pixbuf

                self.feed_key += 1
                self.feed_elems[self.feed_key] = element

                piter = treestore.append(piter, [pixbuf,
                    self._get_name(element), self.feed_key, 0])


            elif element.get('type') == "folder":

                pixbuf = self.dir_pixbuf
                
                self.cat_key += 1
                self.cat_elems[self.cat_key] = element

                piter = treestore.append(piter, [pixbuf,
                    self._get_name(element), self.cat_key, 1])
        
                        

            for child in element:
                trav(child, piter)


        root_outline = self.get_feed_outlines()
        if root_outline != None:
            for child in root_outline:
                trav(child)

        ###
        for path in expanded_rows:
            self.treeview.expand_row(path, False)

    def import_feeds(self, filename):
        
        new_tree = ET.parse(filename)

        if self.tree:
            for child in self.tree.getroot():
                if child.tag == "body":
                    body = child
            
            
            for child in new_tree.getroot():
                if child.tag == "head":
                    for sub in child:
                        if sub.tag == "title":
                            t = sub.text

            ## Create new ET.Element 

            new = {}
            new['title'] = t
            new['text'] = t
            new['description'] = t
            new['type'] = "folder"
            new['expanded'] = "false"

            ### imported is an ElementTree element that contains
            ### all the feeds being imported

            imported = ET.SubElement(body, "outline", new)

            for child in new_tree.getroot():
                ### rename opml document
                if child.tag == "head":
                    for sub in child:
                        if sub.tag == "title":
                            sub.text = "Rss Notify Feed List"

                if child.tag == "body":
                    for sub in child:

                        if sub.tag == "outline":
                            imported.append(sub)

                            
        else:
            self.tree = new_tree
            root = self.tree.getroot()
            for child in root:
                for sub in child:
                    if sub.tag == "title":
                        sub.text = "Rss Notify Feed List"

        ### In case outline element does not have a 
        ### "type" attrib
        def add_type_attrib(element):
            if element.tag == "outline":
                if not element.get("type", None):
                    if element.getchildren():
                        element.set("type","folder")
                    else:
                        element.set("type","rss")
                        self.feed_elems[self.feed_key] = element
                        self.feed_key += 1

            for child in element:
                add_type_attrib(child)


        add_type_attrib(self.tree.getroot())
        
        self.save_feeds()

        self.emit("feed-list-changed")
        return True


    def export_feeds(self, filename):
        """ Writes all the feeds to filename
        in opml format.
        """

        def indent(elem, level=0):
            i = "\n" + level*"  "
            if len(elem):
                if not elem.text or not elem.text.strip():
                    elem.text = i + "  "
                for e in elem:
                    indent(e, level+1)
                    if not e.tail or not e.tail.strip():
                        e.tail = i + "  "
                if not e.tail or not e.tail.strip():
                    e.tail = i
            else:
                if level and (not elem.tail or not elem.tail.strip()):
                    elem.tail = i

        indent(self.tree.getroot())
        self.tree.write(filename, "UTF-8")


    def save_feeds(self):
        self.export_feeds(self.opml_file)


